package serial;

/*
 * @(#)SimpleWrite.java	1.12 98/06/25 SMI
 * 
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 * 
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license
 * to use, modify and redistribute this software in source and binary
 * code form, provided that i) this copyright notice and license appear
 * on all copies of the software; and ii) Licensee does not utilize the
 * software in a manner which is disparaging to Sun.
 * 
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */
import gnu.io.CommPortIdentifier;
import gnu.io.NoSuchPortException;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.UnsupportedCommOperationException;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;

/**
 * Class declaration
 * 
 * 
 * @author
 * @version 1.10, 08/04/00
 */
public class SerialComm {
	private CommPortIdentifier portId;
	private OutputStream outputStream;
	private final String defaultPort;
	private Enumeration portList;
	private SerialPort serialPort;
	private boolean portOpen;
	
	public SerialComm(){
		this.defaultPort = "COM1";
		try {
			CommPortIdentifier cpi = CommPortIdentifier.getPortIdentifier(this.defaultPort);
			this.portId = cpi;
		} catch (NoSuchPortException e) {
			System.out.println("porta COM1 inexistente");
			e.printStackTrace();
		}
	}
	
	public void open() {
		try {
			serialPort = (SerialPort) portId.open("SerialComm", 2000);
			this.portOpen = true;
			serialPort.setSerialPortParams(2400, SerialPort.DATABITS_8,SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
			
			try {
				serialPort.notifyOnOutputEmpty(true);
			} catch (Exception e) {
				throw new IllegalArgumentException("NotifyOnOutputEmpty Error");
			}
			
		} catch (PortInUseException e) {
			throw new IllegalArgumentException("Port in use.");
		} catch (UnsupportedCommOperationException e) {
			throw new IllegalArgumentException("Operao no suportada");
		}
	}
	
	private OutputStream getOutputStream() {
		if (!portOpen) {
			throw new IllegalArgumentException(
					"Porta no foi aberta anteriormente!");
		}
		try {
			return serialPort.getOutputStream();
		} catch (IOException e) {
			return null;
		}
	}
	
	private BufferedInputStream getInputStream() {
		if (!portOpen) {
			throw new IllegalArgumentException(
			"Porta no foi aberta anteriormente!");
		}
		try {
			return new BufferedInputStream(serialPort.getInputStream());
		} catch (IOException e) {
			return null;
		}
	}
	
	public String getPortName(){
		if (!portOpen) {
			throw new IllegalArgumentException(
					"Porta no foi aberta anteriormente!");
		}
		return serialPort.getName();
	}
	public void writeMsg(String msgString){
		if (!portOpen) {
			throw new IllegalArgumentException(
					"Porta no foi aberta anteriormente!");
		}
		try {
			this.getOutputStream().write(msgString.getBytes());
			Thread.sleep(1000);//Espera a msg ser enviada.
		} catch (IOException e) {
			throw new IllegalArgumentException("Erro ao escrever na porta!");
		} catch (InterruptedException e) {
			throw new IllegalArgumentException("Erro ao esperar dado ser escrito na porta!");
		}
	}
	
	public void close(){
		serialPort.close();
	}

	public Byte readReturn() {
		byte[] abyte = new byte[1];
		try {
			int read;
			while((read = this.getInputStream().read(abyte) )==-1){}
			return abyte[0];
		} catch (IOException e) {
			e.printStackTrace();
		}
		throw new Error("Erro ao ler da serial");
	}
	

}
