package vplotter;

import java.util.ArrayList;
import java.util.List;

import org.apache.batik.parser.DefaultPathHandler;
import org.apache.batik.parser.ParseException;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;
import vplotter.model.Bezier;
import vplotter.model.Line;
import vplotter.model.Point;

public class MyPathHandler extends DefaultPathHandler {

	private Point actual = new Point();
	private List<Line> list = new ArrayList<Line>();

	@Override
	public void linetoHorizontalRel(float x) throws ParseException {
		System.out.println("linetoHorizontalRel(" + x + ")");
		Point finalPoint = new Point(actual.x + x, actual.y);
		list.add(new Line(actual.clone(), finalPoint));
		this.actual.copy(finalPoint);
		super.linetoHorizontalRel(x);
	}

	@Override
	public void linetoHorizontalAbs(float x) throws ParseException {
		System.out.println("linetoHorizontalAbs(" + x + ")");
		Point finalPoint = new Point(x, actual.y);
		list.add(new Line(actual.clone(), finalPoint));
		this.actual.copy(finalPoint);
		super.linetoHorizontalAbs(x);
	}

	@Override
	public void startPath() throws ParseException {
		System.out.println("startPath()");
		super.startPath();
	}

	@Override
	public void linetoVerticalAbs(float y) throws ParseException {
		System.out.println("linetoVerticalAbs(" + y + ")");
		Point finalPoint = new Point(actual.x, y);
		list.add(new Line(actual.clone(), finalPoint));
		this.actual.copy(finalPoint);
		super.linetoVerticalAbs(y);
	}

	@Override
	public void linetoVerticalRel(float y) throws ParseException {
		System.out.println("linetoVerticalRel(" + y + ")");
		Point finalPoint = new Point(actual.x, actual.y + y);
		list.add(new Line(actual.clone(), finalPoint));
		this.actual.copy(finalPoint);
		super.linetoVerticalRel(y);
	}

	@Override
	public void endPath() throws ParseException {
		System.out.println("endPath()");
		super.endPath();
	}

	@Override
	public void arcAbs(float rx, float ry, float axisRotation,
			boolean largeArcFlag, boolean sweepFlag, float x, float y)
			throws ParseException {
		System.out.println("arcAbs(" + rx + ", " + ry + "," + axisRotation
				+ ", " + largeArcFlag + "," + sweepFlag + ", " + x + ", " + y
				+ ")");
		throw new NotImplementedException();
	}

	@Override
	public void arcRel(float rx, float ry, float axisRotation,
			boolean largeArcFlag, boolean sweepFlag, float x, float y)
			throws ParseException {
		System.out.println("arcRel(" + rx + "," + ry + "," + axisRotation
				+ ", " + largeArcFlag + "," + sweepFlag + ", " + x + "," + y
				+ ")");
//		throw new NotImplementedException();
	}

	@Override
	public void curvetoCubicAbs(float x1, float y1, float x2, float y2,
			float x, float y) throws ParseException {
		System.out.println("curvetoCubicAbs(" + x1 + "," + y1 + "," + x2 + ", "
				+ y2 + "," + x + ", " + y + ")");
		Point current = actual.clone();
		Point toPoint = new Point(x, y);
		Point controlBeg = new Point(x2, y2);
		Point controlEnd = new Point(x1, y1);

		Bezier bezier = new Bezier(current, toPoint, controlBeg, controlEnd);
		list.addAll(bezier.getLines());
		this.actual.copy(toPoint);
	}
	@Override
	public void curvetoCubicRel(float x1, float y1, float x2, float y2,
			float x, float y) throws ParseException {
		System.out.println("curvetoCubicRel(" + x1 + "," + y1 + "," + x2 + ", "
				+ y2 + "," + x + ", " + y + ")");
		Point current = actual.clone();
		Point toPoint = new Point(current.x+x, current.y+y);
		Point controlBeg = new Point(current.x+x2, current.y+y2);
		Point controlEnd = new Point(current.x+x1, current.y+y1);

		Bezier bezier = new Bezier(current, toPoint, controlBeg, controlEnd);
		list.addAll(bezier.getLines());
		this.actual.copy(toPoint);
	}

	@Override
	public void curvetoCubicSmoothAbs(float x2, float y2, float x, float y)
			throws ParseException {
		System.out.println("curvetoCubicSmoothAbs(" + x2 + "," + y2 + "," + x
				+ ", " + y + ")");
//		throw new NotImplementedException();
	}

	@Override
	public void curvetoCubicSmoothRel(float x2, float y2, float x, float y)
			throws ParseException {
		System.out.println("curvetoCubicSmoothRel(" + x2 + "," + y2 + "," + x
				+ ", " + y + ")");
//		throw new NotImplementedException();
	}

	@Override
	public void curvetoQuadraticAbs(float x1, float y1, float x, float y)
			throws ParseException {
		System.out.println("curvetoQuadraticAbs(" + x1 + "," + y1 + "," + x
				+ ", " + y + ")");
//		throw new NotImplementedException();
	}

	@Override
	public void curvetoQuadraticRel(float x1, float y1, float x, float y)
			throws ParseException {
		System.out.println("curvetoQuadraticRel(" + x1 + "," + y1 + "," + x
				+ ", " + y + ")");
//		throw new NotImplementedException();
	}

	@Override
	public void closePath() throws ParseException {
		System.out.println("closePath()");
	}

	@Override
	public void curvetoQuadraticSmoothAbs(float x, float y)
			throws ParseException {
		System.out.println("curvetoQuadraticSmoothAbs(" + x + "," + y + ")");
//		throw new NotImplementedException();
	}

	@Override
	public void curvetoQuadraticSmoothRel(float x, float y)
			throws ParseException {
		System.out.println("curvetoQuadraticSmoothRel(" + x + "," + y + ")");
//		throw new NotImplementedException();
	}

	@Override
	public void linetoAbs(float x, float y) throws ParseException {
		System.out.println("linetoAbs(" + x + "," + y + ")");
		Point finalPoint = new Point(x, y);
		list.add(new Line(actual.clone(), finalPoint));
		this.actual.copy(finalPoint);

	}

	@Override
	public void linetoRel(float x, float y) throws ParseException {
		System.out.println("linetoRel(" + x + "," + y + ")");
		Point finalPoint = new Point(x + actual.x, y + actual.y);
		list.add(new Line(actual.clone(), finalPoint));
		this.actual.copy(finalPoint);
		super.linetoRel(x, y);
	}

	@Override
	public void movetoAbs(float x, float y) throws ParseException {
		System.out.println("movetoAbs(" + x + "," + y + ")");

		this.actual.x = x;
		this.actual.y = y;

		super.movetoAbs(x, y);
	}

	@Override
	public void movetoRel(float x, float y) throws ParseException {
		System.out.println("movetoAbs(" + x + "," + y + ")");

		this.actual.x += x;
		this.actual.y += y;

		super.movetoRel(x, y);
	}

	public List<Line> getList() {
		return list;
	}
}