package vplotter;

import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.batik.parser.PathParser;
import org.apache.batik.parser.TransformListParser;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import serial.SerialComm;
import sun.text.normalizer.CharTrie.FriendAgent;
import vplotter.model.Line;
import vplotter.model.Point;

/*
 * Calculo para encontrar o movimento dos motres
 * 
 * O perimetro do motor  12mm * pi = 37,69mm
 * 
 * O perimetro tem 360  Graus.
 * O motor se movimenta a passos de 1.8 graus
 * Regra de tres
 *  37,69      360
 *    x        1,8
 * 
 * x = 0,18845 mm
 * para 1 milimetro o motor deve dar 5 passos
 * 
 * 
 * 
 * distancia d = 97cm = 970 mm
 * 
 */

public class VPlotter {
	
	private int D = 360; // em milimetros
	private double variacaoPorPasso = 0.18845;// em milimetros
	
	private double resoluction = 1;
	private Point actual = new Point(0, 0); // ponto atual do plotter
	private SerialComm sc;
	private boolean invertA = false;
	private boolean invertB = false;

	public static void main(String[] args) {
		VPlotter plotter = new VPlotter();
		plotter.setInvertA(true);
		plotter.setInvertB(false);
		plotter.print("imagem.bmp");
	}
	
	public VPlotter(){
		configureSerialPort();
	}
	
	private void configureSerialPort() {
		System.out.println("Abrindo interface serial");
		this.sc = new SerialComm();
		sc.open();
		new Thread(new Runnable(){
			
			@Override
			public void run() {
				while(true){
					Byte readReturn = sc.readReturn();
					System.out.println(readReturn);
				}
			}
		}).start();
	}

	public void print(String file){
		System.out.println("Desenhando imagem " + file);
		List<Line> list = transform(file);
		new TestWindow(list);//TODO fica aqui isso?
		printLines(list);
	}

	protected void printLines(List<Line> list) {
		System.out.println("Plotter envio msg de ready");
		for (Line line : list) {
			System.out.println("Calculando cordenada para linha: "+line);
			
			Point p1 = line.p1;
			System.out.println("Primeiro ponto "+p1);
			List<MovementMsg> msgMovement = getMsgMovement(actual,p1);
			sendToPlotter(msgMovement);
			actual.copy(p1); //agora ponto atual  o proximo pontos
						
			Point p2 = line.p2;
			System.out.println("Segundo ponto "+p2);
			List<MovementMsg> msgMovement2 = getMsgMovement(actual,p2);
			sendToPlotter(msgMovement2);
			actual.copy(p2);
			
		}
	}
	protected void sendToPlotter(List<MovementMsg> movements){
		for (MovementMsg movementMsg : movements) {
			this.sendToPlotter(movementMsg);
		}
	}	
	protected void sendToPlotter(MovementMsg msgMovement) {
		
		msgMovement.setInvertLeft(this.invertA);
		msgMovement.setInvertRight(this.invertB);
		
		
		
		
		msgMovement.setLeftChange(msgMovement.getLeftChange() * resoluction);
     	msgMovement.setRigthChange(msgMovement.getRigthChange()* resoluction);
		
		String stringMsg = msgMovement.getStringMsg();
		System.out.print("Enviando para o plotter a msg:  ");
		System.out.println(msgMovement);
		this.sc.writeMsg(stringMsg);
	}

	public List<MovementMsg> getMsgMovement(Point firstPoint,Point lastPoint){
		
		double dx = getX(firstPoint, lastPoint);
		double dy = getY(firstPoint, lastPoint);

		System.out
				.println("Variao do ponto inicial at o ponto final  de dx="
						+ dx + " dy=" + dy);
		
		double variationX = dx*  0.05;
		double variationY = dy*  0.05;
		
		System.out.println("Usando a variao para calculos de linhas de varx="+variationX+" vary="+variationY);
		
		Point tempPoint = new Point();
		tempPoint.copy(firstPoint);
		System.out.println("Ponto inicial  " + tempPoint);
		
		List<MovementMsg> listMM = new ArrayList<MovementMsg>();
		while(Math.abs(tempPoint.x)<Math.abs(dx)|| Math.abs(tempPoint.y)<Math.abs(dy)){
			Point p = new Point(tempPoint.x+variationX,tempPoint.y+variationY);
			
			System.out.println("Proximo ponto :"+p);
			
			double dx2 = getX(tempPoint, p);
			double dy2 = getY(tempPoint, p);
			
			System.out.println("Distancia at incial dx="+dx2+" dy="+dy2);
			
			double da = getDa(tempPoint.x, tempPoint.y, dx2, dy2);
			double db = getDb(tempPoint.x, tempPoint.y,dx2, dy2);
			
			System.out.println("Variao de da="+da+" db="+db);

			MovementMsg mm = new MovementMsg();
			mm.setLeftChange(da);
			mm.setRigthChange(db);
			listMM.add(mm);
			tempPoint = p;
		}
			
		return listMM;
	}
	
	/**
	 * Calcula a variao em x
	 * @param inicial
	 * @param finall
	 * @return
	 */
	private double getX(Point inicial,Point finall){
		return finall.x - inicial.x;
	}
	
	/**
	 * Calcula a variao em y
	 * @param inicial
	 * @param finall
	 * @return
	 */
	private double getY(Point inicial,Point finall){
		return finall.y - inicial.y;
	}
	
	/**
	 * Calcula a variao na linha A para um valor de x e y e para a variao em x e y
	 * @param x TODO
	 * @param y TODO
	 * @param dx
	 * @param dy
	 * @param x
	 * @param y
	 */
	private double getDa(double x,double y, double dx, double dy){
		double xpow = Math.pow(x, 2);
		double ypow = Math.pow(y, 2);
		
		double varx = (2 * x)/(2 * Math.sqrt(xpow+ypow));
		double vary = (2 * y)/(2 * Math.sqrt(xpow+ypow));
		
		return (varx* dx) + (vary * dy);
	}

	/**
	 * Calcula a variao na linha B para um valor de x e y e para a variao em x e y
	 * 
	 * @param x
	 * @param y
	 * @param dx
	 * @param dy
	 * @param y 
	 * @param x 
	 */
	private double getDb(double x, double y,double dx,double dy){
		double xpow = Math.pow(D - x, 2);
		double ypow = Math.pow(y, 2);
		
		double varx = (2 * (D-x) * -1)/(2 * Math.sqrt(xpow+ypow));
		double vary = (2 * y)/(2 * Math.sqrt(xpow+ypow));
		
		return (varx* dx) + (vary * dy);
	}

	
	private List<Line> transform(String image) {
		try {
			System.out.println("Fanzendo transformao da imagem para linhas.");
			String[] split = image.split("\\.");
			String output = split[0]+".svg";
			convertImg(image, output);
			List<Line> list = parserImagem(output);
			return list;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	private List<Line> parserImagem(String output) throws ParserConfigurationException,
			SAXException, FileNotFoundException, IOException {
		System.out.println("Fazendo parser da imagem " + output);
		SAXParserFactory parserFactory = SAXParserFactory.newInstance();
		SAXParser saxParser = parserFactory.newSAXParser();
		File f = new File(output);
		FileInputStream fis = new FileInputStream(f);
		
		final List<Line> list = new ArrayList<Line>();
		saxParser.parse(fis, new DefaultHandler() {
			public MyTransformListHandler transformListHandler = null;

			@Override
			public void startElement(String uri, String localName,
					String name, Attributes attributes) throws SAXException {

				if ("g".equals(name)) {

					String transform = attributes.getValue("transform");
					if (transform != null) {
						TransformListParser tlp = new TransformListParser();
						transformListHandler = new MyTransformListHandler();
						tlp.setTransformListHandler(transformListHandler);
						tlp.parse(transform);
					}
				}

				if ("path".equals(name)) {
					String path = attributes.getValue("d");
					PathParser pp = new PathParser();
					MyPathHandler pathHandler = new MyPathHandler();
					pp.setPathHandler(pathHandler);
					pp.parse(path);

					List<Line> lineList = pathHandler.getList();
					if (transformListHandler != null) {
						System.out.println("Fazendo as transformes para as linhas");
						for (Line line : lineList) {
//						System.out.println("Fazendo as transformes necessrias para linha "+line);
						if (transformListHandler.getScaleX() != null ||transformListHandler.getScaleY() != null) {
								AffineTransform af = new AffineTransform();
								Point2D pStart = new Point2D.Double(line.p1.x, line.p1.y);
								Point2D pEnd = new Point2D.Double();

								af.scale(transformListHandler.getScaleX(),
										transformListHandler.getScaleY());
								af.transform(pStart, pEnd);
								
								line.p1.x = pEnd.getX();
								line.p1.y = pEnd.getY();
								
								AffineTransform af2 = new AffineTransform();
								Point2D pStart2 = new Point2D.Double(line.p2.x, line.p2.y);
								Point2D pEnd2 = new Point2D.Double();

								af2.scale(transformListHandler.getScaleX(),
										transformListHandler.getScaleY());
								af2.transform(pStart2, pEnd2);
								
								line.p2.x = pEnd2.getX();
								line.p2.y = pEnd2.getY();
//								System.out.println("Linha sofreu transformao de escala para "+line);
							}
							
							line.p1.x += transformListHandler.getTranslateX();
							line.p1.y += transformListHandler.getTranslateY();
							line.p2.x += transformListHandler.getTranslateX();
							line.p2.y += transformListHandler.getTranslateY();
//							System.out.println("Linha sofreu transformao de translao para "+line);
						}
					}
					System.out.println("Fim das transformao nas linhas");
					list.addAll(lineList);
				}
			}
		});
		fis.close();
		return list;
	}

	private void convertImg(String imageInput, String imageOutput)
			throws IOException {
		System.out.println("Usando mkbimap...");
		String command1 = ".\\potrace\\mkbitmap.exe -f 2 -s 2 -o tmp.bmp "
				+ imageInput;
		System.out.println("Executando comando:" + command1);
		Process exec1 = Runtime.getRuntime().exec(command1);
		InputStream inputStream1 = exec1.getInputStream();

		int read1 = 0;
		while ((read1 = inputStream1.read()) != -1) {
			System.out.print((char) read1);
		}
		inputStream1.close();

		System.out.println("Usando potrace...");

		String outputName2 = imageOutput;
		String inputName2 = "tmp.bmp";
//		String command2 = ".\\potrace\\potrace.exe  --opttolerance 2 --alphamax 0.0001 --longcurve -s -o "
//				+ outputName2 + " " + inputName2;
		String command2 = ".\\potrace\\potrace.exe -s -o "
			+ outputName2 + " " + inputName2;
		System.out.println("Executando comando:" + command2);
		Process exec2 = Runtime.getRuntime().exec(command2);
		InputStream inputStream = exec2.getInputStream();

		int read2 = 0;
		while ((read2 = inputStream.read()) != -1) {
			System.out.print((char) read2);
		}
		inputStream.close();
		System.out.println("Imagem " + imageInput + " convertida para "
				+ imageOutput);
	}
	public void setInvertA(boolean invertA) {
		this.invertA = invertA;
	}
	public void setInvertB(boolean invertB) {
		this.invertB = invertB;
	}

	public Point getActual() {
		Point point = new Point();
		point.copy(this.actual);
		return point;
	}
	public void setAsFirstPoint(){
		this.actual.x=0;
		this.actual.y=0;
	}
}
