package model;

import java.awt.Graphics2D;

import vplotter.model.Area;
import vplotter.model.Line;
import vplotter.model.Point;

public class PlotterModel {
	private Line lineA;
	private Line lineB;
	private double width;
	private double larguraCaixa = 50;
	private double alturaCaixa = 50;
	private boolean canetaAbaixada;
	private final Area area;
	private final float passoMotor = 1.8f;
	private final float diametroRodaMotor = 20; //em milimetros mm
	private final double milimetrosPorPasso;

	public PlotterModel(Area area) {
		this.area = area;
		Point aEndPoint = new Point((area.largura / 2) - (larguraCaixa / 2),
				(area.altura / 2) - (alturaCaixa / 2));
		Point aStartPoint = new Point(0, 0);
		this.lineA = new Line(aStartPoint, aEndPoint);

		Point bStartPoint = new Point((area.largura / 2) + (larguraCaixa / 2),
				(area.altura / 2) - (alturaCaixa / 2));
		Point bEndPoint = new Point(this.area.largura, 0);
		this.lineB = new Line(bStartPoint, bEndPoint);

		this.width = area.largura;
		
		//Permetro
		// A extenso da circunferncia, ou seja, seu permetro  pode ser calculada atravs da equao:
		// 2*Pi*Raio ou Pi*Diametro
		
		double perimetroRodaMotor = diametroRodaMotor * Math.PI;
		
		//Regra de 3 para descobrir quantos mm a roda anda para cada passo do motor
		// 360            perimetroRodaMotor
		// passoMotor     x
		
		milimetrosPorPasso = (passoMotor * perimetroRodaMotor)/360;
			

	}

	public void draw(Graphics2D g) {
		// Desenha Linha 1
		g.drawLine((int) lineA.p1.x, (int) lineA.p1.y, (int) lineA.p2.x,
				(int) lineA.p2.y);
		// Desenha Linha 2
		g.drawLine((int) lineB.p1.x, (int) lineB.p1.y, (int) lineB.p2.x,
				(int) lineB.p2.y);

		// desenha caixa, inicia no ponto final de A
		g.drawRect((int) lineA.p2.x, (int) lineA.p2.y, (int) larguraCaixa,
				(int) alturaCaixa);
	}

	public Area getArea() {
		return this.area;
	}

	public double getLarguraLinhaA() {
		double a = this.lineA.p2.x - this.lineA.p1.x;
		double b = this.lineA.p2.y - this.lineA.p1.y;

		return calculeHipotenusa(a, b);
	}

	public double getLarguraLinhaB() {
		double a = this.width - (this.lineA.p2.x - this.lineA.p1.x);
		double b = this.lineA.p2.y - this.lineA.p1.y;

		return calculeHipotenusa(a, b);
	}

	private double calculeHipotenusa(double a, double b) {
		double dx = a;
		double dx2 = Math.pow(dx, 2);

		double dy = b;
		double dy2 = Math.pow(dy, 2);

		double largura = Math.sqrt(dx2 + dy2);

		return largura;
	}

	public boolean isCanetaAbaixada() {
		return canetaAbaixada;
	}

	public void setCanetaAbaixada(boolean canetaAbaixada) {
		this.canetaAbaixada = canetaAbaixada;
	}

	public Line getLineA() {
		return lineA;
	}

	public Line getLineB() {
		return lineB;
	}

	public double getWidth() {
		return width;
	}

	public double getLarguraCaixa() {
		return larguraCaixa;
	}

	public double getAlturaCaixa() {
		return alturaCaixa;
	}
	
	/**
	 * Move o motor no sentido horario, puxando a corda esquerda
	 */
	public void passoHorarioMotorEsquerda(){
		//Altera a possio da linha a,b altera a possio da linha b
		//Altera a possio de todo o sistema
		
		
		//vou variar o tamanho da linha a em 1

//		double dx = 1/(Math.sqrt(a));
//		double dy = 2*1;
		
//		this.lineA.p2.x-=dx;
//		this.lineA.p2.x-=dy;
//		
//		this.lineA.p2.y-=dx;
//		this.lineA.p2.y-=dy;
//		
//		this.lineB.p1.x-=dx;
//		this.lineB.p1.x-=dy;
//		
//		this.lineB.p1.y-=dx;
//		this.lineB.p1.y-=dy;
	}
	
	/**
	 * Move o motor no sentido horario, puxando a corda direita
	 */
	public void passoHorarioMotorDireita(){
		//Diminui o tamanho da linha a,b
	}
	
	/**
	 * Move o motor no sentido anti-horario, soltando a corda esquerda
	 */
	public void passoHorarioAntiMotorEsquerda(){
		double dx = 2*1;
		double dy = 2*1;
		
		this.lineA.p2.x+=dx;
		this.lineA.p2.x+=dy;
		
		this.lineA.p2.y+=dx;
		this.lineA.p2.y+=dy;
		
		this.lineB.p1.x+=dx;
		this.lineB.p1.x+=dy;
		
		this.lineB.p1.y+=dx;
		this.lineB.p1.y+=dy;
	}
	
	/**
	 * Move o motor no sentido anti-horario, soltando a corda direita
	 */
	public void passoHorarioAntiMotorDireita(){
		//Aumenta o tamanho da linha a,b
	}
}
